# frozen_string_literal: true

require 'puppet/resource_api'
require 'puppet/type/f5'

klass = Puppet::ResourceApi.register_type(Puppet::Type::F5.definition({
  name: 'f5_addresslist',
  docs: <<-EOS,
@summary a f5_addresslist type
@example
f5_addresslist { 'foo':
  ensure => 'present',
}

This type provides Puppet with the capabilities to manage ...

If your type uses autorequires, please document as shown below, else delete
these lines.
**Autorequires**:
* `Package[foo]`
EOS
  baseurl: 'tm/security/firewall/address-list',
  attributes: {
    address_lists: {
      type: <<-EOS,
        Optional[
          Array[
            Struct[
              {
                NotUndef[name]      => String[1],
                NotUndef[partition] => String[1],
                Optional[sub_path]  => String[1],
              }
            ]
          ]
        ]
      EOS
      desc: 'List of references to other address-lists.',
      struct_skip_keys: [:nameReference],
      canonicalize: [],
    },
    addresses: {
      type: 'Optional[Array[Struct[{ NotUndef[name] => String }]]]',
      desc: 'List of IPs and IP ranges.',
      canonicalize: [],
    },
    fqdns: {
      type: 'Optional[Array[Struct[{ NotUndef[name] => String[1] }]]]',
      desc: 'List of FQDNs.',
      canonicalize: [],
    },
    geo: {
      type: 'Optional[Array[Struct[{ NotUndef[name] => String[1] }]]]',
      desc: 'List of F5 specific Regions\Country codes.',
      canonicalize: [],
    }
  },
}))

require 'puppet_x/f5/comparer'

klass.autobefore(:f5_addresslist) do
  autos = []
  # Refresh the state of the resource
  refresh_current_state unless @rsapi_current_state
  # If we are not purging this resource needs to be deleted before referenced resources
  if purging?
    autos = @rsapi_current_state[:address_lists].map do |resource_ref|
      "/#{resource_ref['partition']}/#{resource_ref['sub_path']}/#{resource_ref['name']}"
    end
  end
  autos
end

class Puppet::Type::F5_addresslist::Addresses
  def property_matches?(current, desired)
    PuppetX::F5::Comparer.compare(current, desired)
  end
end

class Puppet::Type::F5_addresslist::Fqdns
  def property_matches?(current, desired)
    PuppetX::F5::Comparer.compare(current, desired)
  end
end

class Puppet::Type::F5_addresslist::Geo
  def property_matches?(current, desired)
    PuppetX::F5::Comparer.compare(current, desired)
  end
end

class Puppet::Type::F5_addresslist::Address_lists
  def property_matches?(current, desired)
    PuppetX::F5::Comparer.compare(current, desired)
  end
end
