# frozen_string_literal: true

require 'puppet/resource_api'
require 'puppet/type/f5'

Puppet::ResourceApi.register_type(Puppet::Type::F5.definition({
  name: 'f5_node',
  docs: <<-EOS,
@summary An IControlRest type for LTM nodes
@example
f5_node { 'foo':
  ensure => 'present',
}

This type provides Puppet with the capabilities to manage LTM nodes on F5 systems
EOS
  baseurl: 'tm/ltm/node',
  attributes: {
    address: {
      type: 'Optional[String]', # TODO: Optional[Stdlib::IP::Address]
      desc: 'IP address of the node.',
      behaviour: :init_only,
    },
    connection_limit: {
      type:    'Optional[Integer]',
      desc:    'Specifies the maximum number of connections allowed for the node or node address.',
      default: 0,
    },
    dynamic_ratio: {
      type:    'Optional[Integer]',
      desc:    'Sets the dynamic ratio number for the node. Used for dynamic ratio load balancing.',
      default: 1,
    },
    ephemeral: {
      type: 'Optional[Enum["false", "true"]]',
      desc: '',
      default: 'false',
    },
    logging: {
      type: 'Optional[Enum[disabled, enabled]]',
      desc: 'Specifies whether the node’s monitor(s) actions will be logged.',
      default: 'disabled',
    },
    monitor: {
      type: 'Optional[String]',
      desc: 'Specifies the name of the monitor or monitor rule that you want to associate with the node.',
    },
    rate_limit: {
      type: 'Optional[Enum[disabled, enabled]]',
      desc: 'Specifies the maximum number of connections per second allowed for a node or node address.',
      default: 'disabled',
    },
    ratio: {
      type: 'Integer',
      desc: 'Specifies the fixed ratio value used for a node during ratio load balancing.',
      default: 1,
    },
    session: {
      type: 'Optional[String]',
      desc: 'Enables or disables the node for new sessions.',
    },
    state: {
      type: 'Optional[String]',
      desc: 'Marks the node up or down.',
    },
  },
}))
